from essential_generators import Random36
import random
import os


class MarkovTextGenerator():

    startword = "STARTWORD"
    stopword = "STOPWORD"

    def __init__(self, model=None, load_model=True):
        self.grams = {}
        self.chain = {}
        if model is None:
            model_path = os.path.join(os.path.dirname(__file__), 'markov_textgen.json')
        else:
            model_path = model

        if load_model:
            self.load_model(model_path)
            self.chain_list = list(self.chain)

    def gen_word(self, safe_text=False):

        return random.choice(self.chain_list).split()[0]

    def gen_text(self, max_len=500):
        text = random.choice(self.chain_list).split()

        current_bigram = text[0]

        text_append = text.append
        space_join = " ".join
        while len(text) < max_len:
            transition = self._get_weighted_transition(current_bigram)
            if transition is not None:
                text_append(transition)
                current_bigram = space_join([text[-2], text[-1]])
            else:
                current_bigram = random.choice(self.chain_list)
                text += [self.chain[current_bigram]['left'], self.chain[current_bigram]['right']]

        response = " ".join(text).replace("<br/>", "\n")
        return response

    def _get_weighted_transition(self, bigram):
        if bigram not in self.chain:
            return None
        try:
            res = Random36().choices(self.chain[bigram]['transitions'], weights=self.chain[bigram]['weights'])
        except:
            return None

        if len(res) > 0:
            return res[0]

        return None

    def _condition_grams(self):
        bigram_transitions = {}

        for a, b, c in self.grams:

            bigram = "%s %s" % (a, b)
            trans_to = "%s" % (c)

            # print( "%s %s->%s %i" % (a,b,c, self.grams[(a,b,c,)]) )

            if bigram not in bigram_transitions:
                bigram_transitions[bigram] = {}
            if c not in bigram_transitions[bigram]:
                bigram_transitions[bigram][trans_to] = 0
            if 'total' not in bigram_transitions[bigram]:
                bigram_transitions[bigram]['total'] = 0

            bigram_transitions[bigram]['total'] += self.grams[(a, b, c,)]
            bigram_transitions[bigram][trans_to] += self.grams[(a, b, c,)]


        # now, get the percentages for each
        for bigram in bigram_transitions:
            weights = []
            transitions = []
            for trans in bigram_transitions[bigram]:
                if trans != 'total':
                    percent = bigram_transitions[bigram][trans] / bigram_transitions[bigram]['total']
                    bigram_transitions[bigram][trans] = percent

                    weights.append(percent)
                    transitions.append(trans)

            bigram_transitions[bigram]['weights'] = weights
            bigram_transitions[bigram]['transitions'] = transitions
            # also, while we are here..
            words = bigram.split()
            bigram_transitions[bigram]['left'] = words[0]
            bigram_transitions[bigram]['right'] = words[1]

        self.chain = bigram_transitions

        # print(bigram_transitions['+'])

    def _train_on_text(self, text):
        n = 3
        self.grams = {}
        gram_buffer = []
        lines = text.splitlines(keepends=False)

        for line in lines:
            words = line.split()

            if len(words) == 0:
                words = ["<br/>"]

            # words.append("<br/>")
            for word in words:
                # letter = letter.lower()
                gram_buffer.append(word)

                if len(gram_buffer) >= n:
                    as_tuple = tuple(gram_buffer)
                    if as_tuple not in self.grams:
                        self.grams[as_tuple] = 0
                    self.grams[as_tuple] += 1

                    gram_buffer = gram_buffer[1:]

    def train(self, text):
        self._train_on_text(text)
        self._condition_grams()

    def save_model(self, filepath):
        import json
        with open(filepath, 'w', encoding='utf-8') as fp:
            json.dump(self.chain, fp, ensure_ascii=False)

    def load_model(self, filepath):
        import json
        with open(filepath, 'r', encoding='utf-8') as fp:
            self.chain = json.load(fp)

def run_it():
    gen = MarkovTextGenerator()
    for i in range(500):
        #print(i)
        text = gen.gen_text(10)

        #print(text)

if __name__ == '__main__':
    run_it()


import random
import itertools as _itertools
import bisect as _bisect

class Random36(random.Random):
    "Show the code included in the Python 3.6 version of the Random class"


    def choices(self, population, weights=None, *, cum_weights=None, k=1):
        """Return a k sized list of population elements chosen with replacement.

        If the relative weights or cumulative weights are not specified,
        the selections are made with equal probability.

        """
        random = self.random
        if cum_weights is None:
            if weights is None:
                _int = int
                total = len(population)
                return [population[_int(random() * total)] for i in range(k)]
            cum_weights = list(_itertools.accumulate(weights))
        elif weights is not None:
            raise TypeError('Cannot specify both weights and cumulative weights')
        if len(cum_weights) != len(population):
            raise ValueError('The number of weights does not match the population')
        bisect = _bisect.bisect
        total = cum_weights[-1]
        return [population[bisect(cum_weights, random() * total)] for i in range(k)]


import os
import random
from essential_generators import Random36


class MarkovWordGenerator():
    startword = "STARTWORD"
    stopword = "STOPWORD"

    def __init__(self, model=None, load_model=True):
        self.grams = {}
        self.chain = {}

        if model is None:
            model_path = os.path.join(os.path.dirname(__file__) + '/markov_wordgen.json')
        else:
            model_path = model

        if load_model:
            self.load_model(model_path)

    def gen_word(self, max_len=15):
        word = MarkovWordGenerator.startword
        current_bigram = MarkovWordGenerator.startword
        while len(word) < max_len:
            transition = self._get_weighted_transition(current_bigram)

            if transition is not None:
                word += transition
                current_bigram = word[-2:]
            else:
                break

        return word.replace(MarkovWordGenerator.startword, "").replace(MarkovWordGenerator.stopword, "")

    def gen_text(self, max_len=500):

        words = []

        for i in range(1, random.randint(0, max_len)):
            words.append(self.gen_word())

        return " ".join(words)

    def _get_weighted_transition(self, bigram):
        if bigram not in self.chain:
            return None

        res = Random36().choices(self.chain[bigram]['transitions'], weights=self.chain[bigram]['weights'])
        if len(res) > 0:
            return res[0]

        return None

    def _condition_grams(self):
        bigram_transitions = {}
        for a, b, c in self.grams:

            if a == MarkovWordGenerator.startword:
                # not really a bigram in this case
                bigram = "%s" % (a)
                trans_to = "%s%s" % (b, c)

            else:
                bigram = "%s%s" % (a, b)
                trans_to = "%s" % (c)

            # print( "%s%s->%s %i" % (a,b,c, self.grams[(a,b,c,)]) )

            if bigram not in bigram_transitions:
                bigram_transitions[bigram] = {}
            if c not in bigram_transitions[bigram]:
                bigram_transitions[bigram][trans_to] = 0
            if 'total' not in bigram_transitions[bigram]:
                bigram_transitions[bigram]['total'] = 0

            bigram_transitions[bigram]['total'] += self.grams[(a, b, c,)]
            bigram_transitions[bigram][trans_to] += self.grams[(a, b, c,)]

        # now, get the percentages for each
        for bigram in bigram_transitions:
            weights = []
            transitions = []
            for trans in bigram_transitions[bigram]:
                if trans != 'total':
                    percent = bigram_transitions[bigram][trans] / bigram_transitions[bigram]['total']
                    bigram_transitions[bigram][trans] = percent

                    weights.append(percent)
                    transitions.append(trans)

            bigram_transitions[bigram]['weights'] = weights
            bigram_transitions[bigram]['transitions'] = transitions

        self.chain = bigram_transitions

        # print(bigram_transitions['+'])

    def _train_on_text(self, text):
        n = 3
        self.grams = {}
        gram_buffer = []

        for letter in text:
            # letter = letter.lower()

            if letter not in ['\n', ' ']:
                gram_buffer.append(letter)
            else:
                gram_buffer.append(MarkovWordGenerator.stopword)

            if len(gram_buffer) >= n:
                as_tuple = tuple(gram_buffer)
                if as_tuple not in self.grams:
                    self.grams[as_tuple] = 0

                self.grams[as_tuple] += 1

                gram_buffer = gram_buffer[1:]

            if letter in ['\n', ' ', '*']:
                gram_buffer.clear()
                gram_buffer.append(MarkovWordGenerator.startword)

    def train(self, text):
        self._train_on_text(text)
        self._condition_grams()

    def save_model(self, filepath):
        import json
        with open(filepath, 'w', encoding='utf-8') as fp:
            json.dump(self.chain, fp, ensure_ascii=False)

    def load_model(self, filepath):
        import json
        with open(filepath, 'r', encoding='utf-8') as fp:
            self.chain = json.load(fp)





import random
import uuid
import sys
from essential_generators import MarkovWordGenerator, MarkovTextGenerator, StatisticTextGenerator
import re
import string

class DocumentGenerator:
    """
    Fake document generator::

        def nested_item():
            nested_gen = DocumentGenerator()
            nested_gen.set_template({'user': 'email', 'hash': 'gid', 'posts': small_int})
            return nested_gen.gen_doc()

        generator = DocumentGenerator()
        generator.set_template({'id': 'index', 'user': nested_item, 'url': 'url', 'age': 43, 'one_of': ['male', 'female', 'both']})
        print(generator.gen_docs(5))

        Word generation based VERY loosely on

            letter frequencies here: https://en.wikipedia.org/wiki/Letter_frequency
            bigram frequencies here: #http://norvig.com/mayzner.html

    """

    def __init__(self, word_generator=None, text_generator=None, type_map={}):
        self.index = 0
        self.fields = {}
        self.word_cache = []
        self.sentence_cache = []
        self.unique_cache = {}


        if word_generator is None:
            self.word_generator = MarkovWordGenerator()
        else:
            self.word_generator = word_generator

        if text_generator is None:
            self.text_generator = MarkovTextGenerator()
        else:
            self.text_generator = text_generator

        self.type_map = {
            'index': self.index,
            'word': self.gen_word,
            'sentence': self.gen_sentence,
            'paragraph': self.paragraph,
            'email': self.email,
            'guid': self.guid,
            'url': self.url,
            'small_int': self.small_int,
            'integer': self.integer,
            'small_float': self.small_float,
            'floating': self.floating,
            'upc': self.upc,
            'name': self.name,
            'slug': self.slug

        }

        if type_map is not None:
            self.type_map.update(type_map)

    def init_word_cache(self, length=10000):
        """Create a words cache to speed up generation and to limit the number of possible words."""
        for i in range(0, length):
            self.word_cache.append(self.gen_word())

    def init_sentence_cache(self, length=10000):
        """Create a words cache to speed up generation and to limit the number of possible words."""
        for i in range(0, length):
            #print(i)
            self.sentence_cache.append(self.gen_sentence())

    def word(self, safe_word=False):
        """Gets a word from the words cache or generates a new word if the cache is empty"""
        word = None

        while word == None:
            if len(self.word_cache) > 0:
                word = random.choice(self.word_cache)
            else:
                word = self.gen_word()

            if safe_word and re.search(r'\W', word) is not None:
                word = None

        return word


    def sentence(self):
        """Gets a sentence from the word cache or generates a new word if the cache is empty"""
        if len(self.sentence_cache) > 0:
            return random.choice(self.sentence_cache)
        else:
            return self.gen_sentence()


    def gen_chars(self, min=1, max=5):
        population = string.printable
        charlist = [random.choice(population) for i in range(min, max)]
        return "".join(charlist)

    def gen_word(self, safe_word=True):
        word = self.word_generator.gen_word()
        return word

    def gen_sentence(self, min_words=3, max_words=15):
        """Generate a new sentence - will use cached words if existing."""
        sentence = self.text_generator.gen_text(random.randint(min_words, max_words))
        sentence = sentence[0].upper() + sentence[1:]
        sentence = sentence.strip()

        ending = "."
        if random.random() > .95:
            ending = '?'
        elif random.random() > .95:
            ending = '!'

        sentence = re.sub(r'\W+$', ending, sentence)
        return sentence

    def paragraph(self, min_sentences=2, max_sentences=15):
        """Generate a new paragraph - will use cached sentences if existing."""
        paragraph = ""
        for i in range(random.randint(min_sentences, max_sentences)):
            paragraph += self.sentence().capitalize() + ". "

        return paragraph.strip()

    def domain(self):
        """Generate a new domain name."""
        domain = ""
        # https://w3techs.com/technologies/overview/top_level_domain/all
        tld = ['com', 'com', 'com', 'com', 'com', 'org', 'gov', 'biz', 'net', 'tv', 'us', 'edu', 'ru', 'co.uk', 'fr',
               'jp']

        junction = random.random()

        if junction < .75:
            domain = "%s.%s" % (self.word(True), random.choice(tld))
        elif junction < .9:
            domain = "%s.%s.%s" % (self.word(True), self.word(True), random.choice(tld))
        else:
            domain = "%s%i.%s" % (self.word(True), random.randint(1, 999), random.choice(tld))
        return domain

    def email(self):
        """Generate a new email address."""
        email = ""
        junction = random.random()
        domain = self.domain()

        if junction < .75:
            email = "%s@%s" % (self.word(True), self.domain())
        elif junction < .9:
            email = "%s.%s@%s" % (self.word(True), self.word(True), self.domain())
        else:
            email = "%s%i@%s" % (self.word(True), random.randint(1, 999), self.domain())

        return email

    def phone(self):
        """Generate a new email address."""
        return "%i-%i-%s" % (random.randint(200, 999), random.randint(111, 999), str(random.randint(0, 9999)).zfill(4))

    def guid(self):
        """Genrate a new GUID - based on uuid.uuid4"""
        return str(uuid.uuid4())

    def set_template(self, template):
        """Set the template for document generation."""
        self.template = template

    def index(self):
        """Get the next number in the index."""
        self.index += 1
        return self.index

    def url(self):
        """Genrate a new URL"""
        mimes = ['html', 'html', 'html', 'html', 'asp', 'jsp', 'php', 'png', 'jpg', 'gif']
        junction = random.random()
        url = self.domain()

        proto = "http"
        if random.random() < .50:
            proto = "https"

        url = "%s://%s" % (proto, url)

        path_depth = random.randint(0, 4)
        for i in range(path_depth):
            url += "/%s" % self.word(True)

        ending = random.random()
        if ending < .30:
            url = "%s.%s" % (url, random.choice(mimes))
        elif ending < 60:
            url = "%s.%s" % (url, self.slug())

        return url

    def slug(self):
        slug = self.word()
        for i in range(random.randint(2, 6)):
            if (random.random() < .8):
                slug += "-" + self.word(True)
            else:
                slug += str(self.small_int())
        return slug

    def small_int(self):
        """Generate an int between 0 and 99"""
        return random.randint(0, 99)

    def integer(self):
        """Generate an int between 0 and sys.maxsize"""
        return random.randint(0, sys.maxsize)

    def small_float(self):
        """Generate a float between 0 and 1"""
        return random.random()

    def floating(self):
        """Generate an float between 0 and sys.maxsize"""
        return random.random() * sys.maxsize

    def upc(self):
        upc = ""
        for i in range(0, 12):
            upc += str(random.randint(0, 9))
        return upc

    def name(self):
        return "%s %s" % (self.word(True).capitalize(), self.word(True).capitalize())


    def _find_unique(self):
        pass

    def document(self):
        """Generate a document based on the current template"""
        doc = {}

        for field in self.template:
            if isinstance(self.template[field], list):
                doc[field] = random.choice(self.template[field])
            elif isinstance(self.template[field], dict):
                if 'unique' in self.template[field]:
                    if self.template[field]['unique']:
                        found_unique = False
                        if field not in self.unique_cache:
                            self.unique_cache[field] = {}

                        max_tries = 10
                        if 'tries' in self.template[field]:
                            max_tries = self.template[field]['tries']

                        tries = 0
                        val = None
                        while not found_unique:
                            if tries < max_tries:
                                val = self.type_map[self.template[field]['typemap']]()
                            elif tries < max_tries*2:
                                val = self.type_map[self.template[field]['typemap']]() + self.gen_chars()
                            else:
                                val = self.guid()

                            if val not in self.unique_cache[field]:
                                self.unique_cache[field][val] = True
                                found_unique = True

                            tries += 1
                        self.unique_cache[field][val] = True
                        doc[field] = val

                    else:
                        doc[field] = self.type_map[self.template[field]['typemap']]()

            elif self.template[field] in self.type_map:
                doc[field] = self.type_map[self.template[field]]()
            elif callable(self.template[field]):
                doc[field] = self.template[field]()
            else:
                doc[field] = self.template[field]
        return doc

    def documents(self, count):
        """Generate an list of documents"""
        docs = []
        for i in range(count):
            docs.append(self.document())

        return docs



#gen = DocumentGenerator(text_generator=MarkovTextGenerator(), word_generator=MarkovWordGenerator)


from .random_36 import Random36
from .markov_textgen import MarkovTextGenerator
from .markov_wordgen import MarkovWordGenerator
from .statistic_wordgen import StatisticTextGenerator
from .document_generator import DocumentGenerator



from essential_generators import Random36
import random

class StatisticTextGenerator():

    letters = "".join(
        ['a' * 81, 'b' * 14, 'c' * 27, 'd' * 42, 'e' * 127, 'f' * 22, 'g' * 20, 'h' * 60, 'i' * 69, 'j' * 1, 'k' * 7,
         'l' * 40, 'm' * 24, 'n' * 67, 'o' * 75, 'p' * 19, 'r' * 59, 's' * 63, 't' * 90, 'u' * 27, 'v' * 9, 'w' * 23,
         'x', 'y' * 19, 'z'])
    # word lengths up to 9
    word_lens = [1]*3 + [2]*18 + [3]*20 + [4]*15 + [5]*10 + [7]*8 + [8]*6 + [9]*4
    bigrams = ['th'] * 35 + ['he'] * 30 + ['in'] * 24 + ['er'] * 20 + ['an'] * 19 + ['re'] * 18 + ['on'] * 17
    bigrams += ['at'] * 14 + ['en'] * 14 + ['nd'] * 13 + ['ti'] * 13 + ['es'] * 13 + ['es'] * 12 + ['or'] * 12
    bigrams += ['te'] * 12 + ['ed'] * 11 + ['is'] * 11 + ['it'] * 11 + ['al'] * 10 + ['ar'] * 10 + ['st'] * 10

    def __init__(self):
        pass

    def gen_text(self, max_len=500):
        words = []

        for i in range(random.randint(1, max_len)):
            words.append(self.gen_word())

        return " ".join(words)

    def gen_word(self):
        """
        Generate a new word, based VERY loosely on

            letter frequencies here: https://en.wikipedia.org/wiki/Letter_frequency
            bigram frequencies here: #http://norvig.com/mayzner.html
        """

        word = ""
        word_len = random.choice(StatisticTextGenerator.word_lens)

        while len(word) < word_len:
            junction = random.random()
            if junction < .60:
                word += random.choice(StatisticTextGenerator.bigrams)
            elif junction < .80:
                word += random.choice(StatisticTextGenerator.letters) + random.choice(StatisticTextGenerator.letters)
            else:
                word += random.choice(StatisticTextGenerator.letters)

        return word



